//
//	Programao Multithreaded em ambiente Windows NT - uma viso de  Automao
// 
//	Autores: Constantino Seixas Filho/ Marcelo Szuster
//
//	Captulo 6 - Exemplo 2	- Pipes No Nomeados - Servidor
//
//	SERVIDOR: PRG52SERVER.CPP
//
//	Verso: 1.1	28/02/1999
//

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "Exemplo62.h"

#define _CHECKERROR	1	// Ativa funo CheckForError
#include "CheckForError.h"

void ShowDump(char *, int);

int main(int argc, char *argv[])// Receber id do processo pai como parmetro
{								// e handles para os dois pipes
	DWORD dwExitCode = 0;
	char FileName[50]= "";		// Nome do arquivo a ser lido
	HANDLE hFile= 0;			// Handle para arquivo a ser aberto
	DWORD dwBytesRead;		
	DWORD dwBytesWritten;	
	LONG lFilePosLow=0L;		// Offset para posicao corrente do arquivo
	BOOL bStatus;
	char MyBuffer[512];			// Buffer para leitura do arquivo
	FileReadDesc DumpRequest;	// Descritor do pedido de leitura 
	DumpFileAck DumpAck;		// Resposta do servidor para cliente
	DWORD dwLength;
	DWORD dwFatherId;			// Id do processo pai

	// Handle do pipe no sentido Cliente Servidor
	HANDLE hCSReadPipe;			// Handle para operaes de leitura
	// Handle do pipe no sentido Sevidor Cliente
	HANDLE hSCWritePipe;		// Handle para operaes de escrita

	HANDLE hFather;				// Handle do processo pai
	HANDLE hPipe1;				// Handle para o pipe no dominio do cliente
	HANDLE hPipe2;				// Handle para o pipe no domnio do cliente
	HANDLE hMyProcess;			// Handle para o prprio processo


	// Obtm handle para o processo pai
	dwFatherId = atoi(argv[0]);
	hPipe1   = (HANDLE) atoi(argv[1]);
	hPipe2   = (HANDLE) atoi(argv[2]);
	hFather  = OpenProcess(PROCESS_DUP_HANDLE, FALSE, dwFatherId);
	CheckForError(hFather);

	hMyProcess = GetCurrentProcess();

	// printf("hFather= %d  hPipe1=%d  hPipe2=%d\n", hFather, hPipe1, hPipe2);
	
	// Cria handle Cliente-Servidor de leitura no espao do cliente
	bStatus = DuplicateHandle(
		hFather,			// Processo que fornece o handle
		hPipe1,				// Handle a ser copiado
		hMyProcess,			// Processo que receber o handle
		&hCSReadPipe,		// Valor do novo handle
		GENERIC_READ,		// FLAGS
		FALSE,				// Handle no herdvel
		DUPLICATE_CLOSE_SOURCE	// Opes: apaga o handle de origem
		);
	CheckForError(bStatus);
	
	// Cria handle Servidor-Cliente de escrita no espao do servidor
	bStatus = DuplicateHandle(
		hFather,			// Processo que fornece o handle
		hPipe2,				// Handle a ser copiado
		hMyProcess,			// Processo que receber o handle
		&hSCWritePipe,		// Valor do novo handle
		GENERIC_WRITE,		// FLAGS
		FALSE,				// Handle no herdvel
		DUPLICATE_CLOSE_SOURCE	// Opes: apaga o handle de origem
		);
	CheckForError(bStatus);


	do {
		// Recebe pedido de dump via pipe
		bStatus=ReadFile(hCSReadPipe, &DumpRequest, sizeof(DumpRequest), &dwBytesRead, NULL);
		CheckForError(bStatus);
		
		lFilePosLow = DumpRequest.InitialPosition;
		dwLength = DumpRequest.Length;
		
		if (dwLength==0) break;	// Flag para abortar processo servidor.

		if (strcmp(FileName, DumpRequest.FileName) != 0) { 
			strcpy(FileName, DumpRequest.FileName);
			// Abre novo arquivo
			if (hFile != 0) CloseHandle(hFile); 
			hFile= CreateFile(
				FileName,
				GENERIC_READ|GENERIC_WRITE,
				FILE_SHARE_READ|FILE_SHARE_WRITE, // abre para leitura e escrita
				NULL,		// atributos de segurana 
				OPEN_EXISTING,// abre se existir 
				FILE_ATTRIBUTE_NORMAL,
				NULL);		// Template para atributos e flags
			CheckForError(hFile != INVALID_HANDLE_VALUE);
			if (hFile == INVALID_HANDLE_VALUE) {
				DumpAck.Erro = ERRO_ARQUIVO_INEXISTENTE;
				// Envia Acknowledge para Cliente
				WriteFile(hSCWritePipe, &DumpAck, sizeof(DumpAck), &dwBytesWritten, NULL);
				break;
			} // if
		}
		
		// Le registro desejado
		lFilePosLow = SetFilePointer(hFile,lFilePosLow,NULL,FILE_BEGIN);
		bStatus = ReadFile(hFile, &MyBuffer, dwLength, &dwBytesRead, NULL);
		CheckForError(bStatus);
		DumpAck.EndOfFile = (dwBytesRead != dwLength)? TRUE: FALSE;
	
		ShowDump(MyBuffer, (int)dwBytesRead);
		
		// Envia status da leitura para processo cliente
		DumpAck.BytesRead = dwBytesRead;
		DumpAck.Erro = 0;
		
		// Envia Acknowledge para Cliente
		WriteFile(hSCWritePipe, &DumpAck, sizeof(DumpAck), &dwBytesWritten, NULL);
	
	} while (1);

	CloseHandle(hFile);	
	CloseHandle(hFather);
	CloseHandle(hSCWritePipe);
	CloseHandle(hCSReadPipe);
	return EXIT_SUCCESS;
}	// main


void ShowDump(char *lpBuffer, int iLength)
{	int i=0;
	int iColuna;
	int iLinha;
	int iLinhas;
	int iResto;

	iLinhas = iLength / 16;
	iResto = iLength %  16;
	
	// Imprime cabealho
	printf("\n00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F  0 1 2 3 4 5 6 7 8 9 A B C D E F\n");
	printf("================================================================================\n");

	for (iLinha=0; iLinha<iLinhas; iLinha++)
	{	for (iColuna=0; iColuna<16; ++iColuna)
			printf("%02x ", lpBuffer[i++] & 0xFF);
		i-= 16;
		printf(" ");
		for (iColuna=0; iColuna<16; ++iColuna)
			if (isprint(lpBuffer[i]))
				printf("%c ", lpBuffer[i++]);
			else { printf(". "); i++; }
		printf("\n");
	}  // for

	if (iResto > 0) {
		// printa ltima linha
		for (iColuna=0; iColuna<16; ++iColuna)
			if (iColuna < iResto) printf("%02x ", lpBuffer[i++] & 0xFF);
			else printf("   "); 
		i-= iResto;
		printf(" ");
		for (iColuna=0; iColuna<iResto; ++iColuna)
			if (isprint(lpBuffer[i]))
				printf("%c ", lpBuffer[i++]);
				else { printf(". "); i++; }
		printf("\n\n");
	}

} // ShowDump
