//
//	Programao Multithreaded em ambiente Windows NT - uma viso de  Automao
// 
//	Autores: Constantino Seixas Filho/ Marcelo Szuster
//
//	Captulo 6 - Exemplo 6 - Uso de mensagens WM_APP e de MsgWaitForMultipleObjects
//
//	Verso: 1.1	28/11/1999
//


#define  WIN32_LEAN_AND_MEAN
#define  _WIN32_WINNT  0x0400	// Necessrio para ativar novas funes da verso 4
#include <windows.h>
#include <process.h>	// _beginthreadex() e _endthreadex() 
#include <stdio.h>
#include <stdlib.h>

#define _CHECKERROR	1	// Ativa funo CheckForError
#include "CheckForError.h"

#include "bGetFloat.h"
#include "GetOSVersion.h"

DWORD WINAPI ThreadControlador(LPVOID);	// declarao da funo Thread
//BOOL bGetFloat(double *, int );
double Pid(double, double, double, double, double);

// Casting para terceiro e sexto parmetros da funo _beginthreadex
typedef unsigned (WINAPI *CAST_FUNCTION)(LPVOID);
typedef unsigned *CAST_LPDWORD;

#define WM_NOVO_SET_POINT       WM_APP + 0x0001
#define WM_NOVO_KP		        WM_APP + 0x0002
#define WM_NOVO_TI		        WM_APP + 0x0003
#define WM_NOVO_TD		        WM_APP + 0x0004

HANDLE hEvent;
double dSetPoint;


int main(VOID) 
{ 
    HANDLE	hThread;
    DWORD	dwThreadId;
	DWORD	dwExitCode;
	BOOL	bStatus;
	int		iProc;		// Nmero de processadores na mquina
	char	strVer[30];	// Verso do sistema operacional
	BOOL	bRet;

	bRet = GetOSVersion(strVer, &iProc);
	printf("Versao do SO = %s  Processadores = %d\n", strVer, iProc);
	if (!bRet) { // No  NT. No  NT ...
		printf("Falha: Sistema operacional NAO E O NT\n");
		printf("\nAcione uma tecla para terminar\n");
		_getch(); // // Pare aqui, caso no esteja executando no ambiente MDS
		return 0;
	} // if

	// Cria um objeto do tipo evento para sincronizar duas threads
	// O evento ser sinalizado quando a thrad estiver apta a receber mensagens
    hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
 
   	// Cria thread controlador PID
	hThread = (HANDLE) _beginthreadex(
			NULL,
			0,
			(CAST_FUNCTION) ThreadControlador,	// casting necessrio
			0,
			0,
			(CAST_LPDWORD)&dwThreadId	// casting necessrio
		);
	CheckForError(hThread);
	if (hThread) printf("Thread criada Id= %0x \n", dwThreadId);

    // Espera sincronismo pasra iniciar comunicao
    WaitForSingleObject(hEvent, INFINITE);	
	
	for (int cont=0; cont<3 ;++cont) {
		do {
			printf("\nEntre com o novo valor do SetPoint: ");
			// le string ou ESC
			bStatus = bGetFloat(&dSetPoint, 8);
			if (bStatus)  // Entrada vlida
				PostThreadMessage(dwThreadId, WM_NOVO_SET_POINT, NULL, (LPARAM)&dSetPoint);
			// Sleep(50); 
		} while (!bStatus);
	}

	// Encerra thread controladora
	PostThreadMessage(dwThreadId, WM_QUIT, NULL, NULL);

	// Espera controladora terminar
    WaitForSingleObject(hThread, INFINITE);
	GetExitCodeThread(hThread, &dwExitCode);
    CloseHandle(hThread);

	CloseHandle(hEvent);
	
	return EXIT_SUCCESS;
}

 
// Thread que implementa controlador PID
DWORD WINAPI ThreadControlador(LPVOID n)
{
    UNREFERENCED_PARAMETER(n);

    MSG msg;
	DWORD dwRet; 
	double dSetPoint, dKp, dTd, dTi, dPV= 0.0;
	LARGE_INTEGER Preset;
	HANDLE hTimer;			// Handle para Timer
	// Define uma constante para acelerar clculo do atraso e perodo
	const int nMultiplicadorParaMs = 10000; 

    // Cria fila de mensagens
    PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE);

	// Avisa que est apto a receber mensagens
    SetEvent(hEvent);

    // Define timer para gerar evento a cada 1s
	// Cria timer com reset automtico
	hTimer = CreateWaitableTimer(NULL, FALSE, "MyTimer");
	CheckForError(hTimer);

	// Programa o temporizador para que a primeira sinalizao ocorra 2s 
	// depois de SetWaitableTimer 
	// Use - para tempo relativo
	Preset.QuadPart = -(2000 * nMultiplicadorParaMs);
	SetWaitableTimer(hTimer, (const LARGE_INTEGER *)&Preset, 5000, NULL, NULL, FALSE); // T = 2s

	for (; ;)
	{
		dwRet = MsgWaitForMultipleObjects(1, &hTimer, FALSE, INFINITE, QS_POSTMESSAGE);
    
		if (dwRet == WAIT_OBJECT_0) // foi o timer
			Pid(dSetPoint, dKp, dTd, dTi, dPV);
		else { // mudou algum parmetro 
			PeekMessage(&msg, NULL, 0, 0, PM_REMOVE);
			double dParametro= *((double *)msg.lParam);

			switch(msg.message)
			{
			case WM_NOVO_SET_POINT:
				printf("\nControlador: novo SP = %f\n", dParametro);
				dSetPoint = dParametro;
				break;
			case WM_NOVO_KP:
				printf("\nControlador: novo KP = %f\n", dParametro);
				dKp = dParametro;
				break;
			case WM_NOVO_TI:
				printf("\nControlador: novo TI = %f\n", dParametro);
				dTi = dParametro;
			    break;
			case WM_NOVO_TD:
				printf("\nControlador: novo TD = %f\n", dParametro);
				dTd = dParametro;
			case WM_QUIT:
				break;
			default:
				DispatchMessage(&msg);
			} // switch
		} // else
	} // for

	CloseHandle(hTimer);

    return 0;
}


double	Pid(double dSetPoint, double dKp, double dTd, double dTi, double dPV)
{	double dMV;	// varivel manipulada

	printf("> ");

	dMV = 0.0;
	// Calcula varivel manipulada
	return(dMV);
}  // Pid


